From b6117da06f6d72d847985e5f179db62beb0227ca Mon Sep 17 00:00:00 2001
From: David Plowman <david.plowman@raspberrypi.com>
Date: Wed, 17 Feb 2021 18:08:12 +0000
Subject: [PATCH] media: i2c: imx290: Fix up exposure calcuations and
 ranges

Should now correspond exactly to the datasheet.

Signed-off-by: David Plowman <david.plowman@raspberrypi.com>
---
 drivers/media/i2c/imx290.c | 14 +++++++-------
 1 file changed, 7 insertions(+), 7 deletions(-)

--- a/drivers/media/i2c/imx290.c
+++ b/drivers/media/i2c/imx290.c
@@ -50,7 +50,7 @@ enum imx290_clk_index {
 #define IMX290_HMAX_MIN_4LANE 2200 /* Min of 2200 pixels = 60fps */
 #define IMX290_HMAX_MAX 0xffff
 
-#define IMX290_EXPOSURE_MIN 2
+#define IMX290_EXPOSURE_MIN 1
 #define IMX290_EXPOSURE_STEP 1
 #define IMX290_EXPOSURE_LOW 0x3020
 #define IMX290_PGCTRL 0x308c
@@ -584,7 +584,7 @@ static int imx290_set_gain(struct imx290
 static int imx290_set_exposure(struct imx290 *imx290, u32 value)
 {
 	u32 exposure = (imx290->current_mode->height + imx290->vblank->val) -
-						value;
+						value - 1;
 	int ret;
 
 	ret = imx290_write_buffered_reg(imx290, IMX290_EXPOSURE_LOW, 3,
@@ -855,10 +855,10 @@ static int imx290_set_fmt(struct v4l2_su
 		}
 		if (imx290->exposure)
 			__v4l2_ctrl_modify_range(imx290->exposure,
-						 mode->vmax - mode->height,
-						 mode->vmax - 4,
+						 IMX290_EXPOSURE_MIN,
+						 mode->vmax - 2,
 						 IMX290_EXPOSURE_STEP,
-						 mode->vmax - 4);
+						 mode->vmax - 2);
 	}
 
 	*format = fmt->format;
@@ -1345,9 +1345,9 @@ static int imx290_probe(struct i2c_clien
 	imx290->exposure = v4l2_ctrl_new_std(&imx290->ctrls, &imx290_ctrl_ops,
 					     V4L2_CID_EXPOSURE,
 					     IMX290_EXPOSURE_MIN,
-					     mode->vmax - 4,
+					     mode->vmax - 2,
 					     IMX290_EXPOSURE_STEP,
-					     mode->vmax - 4);
+					     mode->vmax - 2);
 
 	imx290->hflip = v4l2_ctrl_new_std(&imx290->ctrls, &imx290_ctrl_ops,
 					  V4L2_CID_HFLIP, 0, 1, 1, 0);
